MODULE bioanalysis

! -------------------------------------------------------------------------
! BIOANALYSIS
!
! John Hemmings - 18th May 2006 
! -------------------------------------------------------------------------
! MODIFICATIONS
! -------------------------------------------------------------------------
! 24th August 2007 (John Hemmings):
!  - corrected calculation of balancing factor limits below current depth
!    of mixed layer - now use local phytoplankton increment rather than
!    that at surface
!  - 'specific_loss' calculation changes to improve accuracy - removed
!    multiplication and subsequent division by phytoplankton conc.
!  - zooplankton upper bound was set using the wrong parameter ('zrfmax'
!    instead of 'zafmax') - now corrected
!  - unit values of 'qrfmax' and 'qafmax' parameters are now handled
!    explicitly to avoid numerical error problems leading to negative
!    nutrient balancing factors
!  - corrected processing for diagnostic 'k_plobound_d' 
!  - 'get_integration_bounds' changed to explicitly set bounds to zero if
!    no integration is to be done (defensive programming)
!  - part of code restructured to facilitate potential modifications to 
!    zooplankton loss fraction calculation in future versions 
! -------------------------------------------------------------------------
!
! Subroutine to complete a biogeochemical analysis for a row once the 
! surface chlorophyll increments are known. 
! Calculates surface and sub-surface balancing increments for the nitrogen 
! pools, DIC and alkalinity and updates tracers.
!
! Surface mixed layer increments (applied down to 'mld') are the  
! phytoplankton increments calculated from the input chlorophyll increments
! plus the balancing increments for the other tracers at the same grid points. 
! These are primary increments.
!
! Below 'mld', the increments are potentially compound increments
! made up of primary and secondary partial increments:
!
!  - primary increments are modified surface increments  
!    (dependent on the local background state).
!
!  - secondary increments are positive nutrient profile-correction 
!    increments, designed to remove sub-surface nutrient minima created by
!    assimilation, plus balancing increments for the non-nutrient pools at the 
!    same grid points. 
!
! A primary increment for a non-phytoplankton nitrogen pool is its 
! negated balancing factor * the phytoplankton increment.
! The balancing factors 'balnut', 'balzoo' and 'baldet' are determined so 
! that they sum to 1 to conserve nitrogen (if possible). 
! Note however that 'balzoo' and 'baldet' can be negative (giving increments 
! that are positively correlated with 'incphy').
!
! A secondary increment for a non-nutrient nitrogen pool is its negated
! balancing factor * the nutrient increment.
! The balancing factors 'balphy', 'balzoo' and 'baldet' are determined so 
! so that they sum to 1 to conserve nitrogen (if possible). 
!
! DIC is incremented to conserve carbon.
! Alkalinity is incremented for consistency with nutrient changes. 
!
! -------------------------------------------------------------------------
! History:
!   2009/05/20    D.J.Lea     Remove m to cm depth conversion  
!                             and allow for spatially varying zdz
!
!   2010/10/07    D.A.Ford    Implement D. Lea's changes above in this
!                             modified (on 2007/08/24) version.
!                             Convert to Fortran90 and enclose in module.
!                             Add j dependence, so it now processes the
!                             entire field in one go, rather than a row
!                             of i and k at a time.
! -------------------------------------------------------------------------
!
! -------------------------------------------------------------------------

  IMPLICIT NONE
  PRIVATE

  PUBLIC bio_analysis
  
  CONTAINS

     subroutine bio_analysis(imt,jmt,km,zdz,i_tracer,modparm, &
     &                       n2be_p,n2be_z,n2be_d,assimparm, &
     &                       timestep,ntstep_assim,kmt,fm, &
     &                       mld,mldmax,incchl,cchl, &
     &                       nbal_active,avphy, &
     &                       gl_active,avpgrow,avploss, &
     &                       subsurf_active,deepneg_active, &
     &                       deeppos_active,nutprof_active, &
     &                       bstate,inc, &
     &                       diag_active,diag, &
     &                       diag_fulldepth_active,diag_fulldepth)

      implicit none

      real mw_carbon
      parameter (mw_carbon=12.01)      ! Molecular weight of carbon

      real incphytiny
      parameter (incphytiny=1.0e-9)    ! Limit for 'incphymin' parameter 

! Local tracer indices
! i.e. positions in the array of external tracer indices 'i_tracer'

      integer n_inc,n_inc_partial
      parameter (n_inc=6)
      parameter (n_inc_partial=4)

      integer nut_t,phy_t,zoo_t,det_t,dic_t,alk_t

      parameter (nut_t=1)
      parameter (phy_t=2)
      parameter (zoo_t=3)
      parameter (det_t=4)
      parameter (dic_t=5)
      parameter (alk_t=6)

! Model parameter indices

      integer grow_sat_parm,c2n_p_parm,c2n_z_parm,c2n_d_parm

      parameter (grow_sat_parm=1)
      parameter (c2n_p_parm=10)
      parameter (c2n_z_parm=11)
      parameter (c2n_d_parm=12)

! Assimilation scheme parameter indices

      integer balnutext_parm,balnutmin_parm
      integer zfracb0_parm,zfracb1_parm
      integer qrfmax_parm,qafmax_parm,zrfmax_parm,zafmax_parm
      integer prfmax_parm,incphymin_parm

      parameter (balnutext_parm=2)
      parameter (balnutmin_parm=3)
      parameter (zfracb0_parm=11)
      parameter (zfracb1_parm=12)
      parameter (qrfmax_parm=13)
      parameter (qafmax_parm=14)
      parameter (zrfmax_parm=15)
      parameter (zafmax_parm=16)
      parameter (prfmax_parm=17)
      parameter (incphymin_parm=18)

! Diagnostic indices 

      integer ubalnut_d
      integer k_nlobound_d,k_nhibound_d
      integer k_plobound_d
      integer k_zlobound_d,k_zhibound_d
      integer k_dlobound_d
      integer kmld_d,kmldmax_d,kmax_d

      parameter (ubalnut_d=1)        ! Unadjusted nutrient balancing factor
      parameter (k_nlobound_d=2)     ! 0 or top level with low N constraint
      parameter (k_nhibound_d=3)     ! 0 or top level with high N constraint 
      parameter (k_plobound_d=4)     ! 0 or top level with low P constraint 
      parameter (k_zlobound_d=5)     ! 0 or top level with low Z constraint
      parameter (k_zhibound_d=6)     ! 0 or top level with high Z constraint
      parameter (k_dlobound_d=7)     ! 0 or top level with low D constraint
      parameter (kmld_d=8)           ! Deepest level for surface increments 
      parameter (kmldmax_d=9)        ! Deepest level in maximum depth mixed
                                     ! layer for assimilation timestep
      parameter (kmax_d=10)          ! Deepest level of analysis updates

! Full depth diagnostic indices

      integer inc1_d,inc2_d,ubalphy_d

      parameter (inc1_d=1)           ! N, P, Z & D increments
      parameter (inc2_d=5)           ! N, P, Z & D increments
      parameter (ubalphy_d=9)        ! Unadjusted phytop. balancing factor

! Arguments

!     Model configuration data 

      integer imt             ! IN Number of points in horizontal 
      integer jmt             ! IN Number of points in vertical
      integer km              ! IN Number of levels in model
      real zdz(imt,jmt,km)    ! IN Depth of base of layer (m)
      integer i_tracer(n_inc) ! IN Array of external tracer indices
      real modparm(*)         ! IN Model parameters
      real n2be_p             ! IN Phytoplankton biomass equivalent factor 
      real n2be_z             ! IN Zooplankton biomass equivalent factor
      real n2be_d             ! IN Detritus biomass equivalent factor 
      real assimparm(*)       ! IN Assimilation scheme parameters
      integer timestep        ! IN Model timestep (s)
      integer ntstep_assim    ! IN Number of timesteps between analyses
                              !    (i.e. in one assimilation timestep)

!     Local configuration

      integer kmt(imt,jmt)    ! IN Number of grid points in each water column
      real fm(imt,jmt,km)     ! IN Land-sea mask

!     Input for phytoplankton update 

      real mld(imt,jmt)       ! IN Current mixing layer depth (m)
      real mldmax(imt,jmt)    ! IN Maximum mixing layer depth for assim. 
                              ! timestep (m)
      real incchl(imt,jmt)    ! IN Chlorophyll increment (mg/m3)
      real cchl(imt,jmt)      ! IN Surface C:chl ratio (mgC/mgChl)

!     Input for nitrogen balancing 

      logical nbal_active     ! IN Switch for nitrogen balancing increments 
      real avphy(imt,jmt)         ! IN Model P for assimilation timestep (mmolN/m3) 

!     Input for growth/loss-based nitrogen balancing scheme 

      logical gl_active       ! IN Switch for growth/loss-based balancing 
      real avpgrow(imt,jmt)   ! IN Model growth rate for assim. timestep (/d) 
      real avploss(imt,jmt)   ! IN Model loss rate for assim. timestep (/d) 

!     Sub-surface increment control data

      logical subsurf_active  ! IN Switch for updates below mixed layer
      logical deepneg_active  ! IN Switch for negative primary increments
                              !    below maximum mixing layer depth 
      logical deeppos_active  ! IN Switch for positive primary increments
                              !    below maximum mixing layer depth 
      logical nutprof_active  ! IN Switch for secondary increments 

!     Tracers & increments

      real bstate(imt,jmt,km,*)   ! IN Tracers - background state 
      real inc(imt,jmt,km,n_inc)  ! OUT Tracer increments 

!     Diagnostics

      logical diag_active     ! IN Switch for diagnostics (depth independent)
      real diag(imt,jmt,*)    ! OUT Diagnostics (depth independent)

      logical diag_fulldepth_active 
                              ! IN Switch for full-depth diagnostics 
      real diag_fulldepth(imt,jmt,km,*)  
                              ! OUT Full-depth diagnostics

! Local variables

!     External tracer indices

      integer nutrient_tracer ! Nutrient
      integer phyto_tracer    ! Phytoplankton
      integer zoo_tracer      ! Zooplankton
      integer dn_tracer       ! Detritus
      integer tco2_tracer     ! Dissolved inorganic carbon (=TCO2)
      integer alk_tracer      ! Alkalinity

!     Model parameter values

      real grow_sat           ! Half-saturation const for N  
      real c2n_p              ! Phytoplankton C:N ratio 
      real c2n_z              ! Zooplankton C:N ratio
      real c2n_d              ! Detritus C:N ratio

!     External assimilation scheme parameter values

      real balnutext          ! Default nutrient balancing factor
      real balnutmin          ! Fraction of phytoplankotn loss to nutrient 
      real zfracb0            ! Base zooplankton fraction of loss to Z & D
      real zfracb1            ! Phytoplankton sensitivity of zooplankton 
                              ! fraction (/mmolN/m3)
      real qrfmax             ! Maximum nutrient limitation reduction factor
      real qafmax             ! Maximum nutrient limitation amplification factor
      real zrfmax             ! Maximum zooplankton reduction factor
      real zafmax             ! Maximum zooplankton amplification factor
      real prfmax             ! Maximum phytoplankton reduction factor 
                              ! (secondary increment only)
      real incphymin          ! Minimum size of non-zero phytoplankton 
                              ! increment (mmolN/m3)

!     Working variables

      real assimperiod            ! Assimilation time step (d)  
      real incphy(imt,jmt)        ! Phytoplankton increment (mmolN/m3)
      integer kmld(imt,jmt)       ! Deepest level fully in current mixed layer 
      integer kmldmax(imt,jmt)    ! Deepest level in maximum depth mixed layer  
      integer kmax(imt,jmt)       ! Deepest level for all increments 
      real expected_nbf(imt,jmt)  ! Growth/loss-based nutrient balancing factor
      real balnutsurf(imt,jmt)    ! Unadjusted nutrient balancing factor at surface
      real inc1(n_inc_partial)    ! Primary increments 
      real inc2(n_inc_partial)    ! Secondary increments
      real balnut                 ! Nutrient balancing factor
      real balphy                 ! Phytoplankton balancing factor
      real balzoo                 ! Zooplankton balancing factor
      real baldet                 ! Detritus balancing factor
      logical done_phy            ! Completion flag for primary increments
                                  ! (phytoplankton plus balancing)
      logical done_nut            ! Completion flag for secondary increments
                                  ! (nutrient plus balancing)
      logical nodeepphyinc        ! Flag for suppressing primary increments
                                  ! below mixed layer depth maximum

      real nut
      real phy 
      real zoo 
      real det
      real bphy
      real bphysurf
      real zfrac
      real phy_zf
      real incphy_zf
      real ploss
      logical ploss_valid
      real anutabove
      real q
      real qbound
      real nutbound
      real zoomin
      real zoomax
      real balnutmax
      real balphymax
      real balzoomax_r
      real balzoomax_a
      real baldetmax
      integer i,j,k,kend,m

      logical diag_nloflag    
      logical diag_nhiflag
      logical diag_ploflag
      logical diag_zloflag
      logical diag_zhiflag
      logical diag_dloflag
      real diag_ubalphy

!!!#if defined key_hadocc

! Get external tracer indices & parameter values

      nutrient_tracer=i_tracer(nut_t)
      phyto_tracer=i_tracer(phy_t)
      zoo_tracer=i_tracer(zoo_t)
      dn_tracer=i_tracer(det_t)
      tco2_tracer=i_tracer(dic_t)
      alk_tracer=i_tracer(alk_t)
      grow_sat=modparm(grow_sat_parm)
      c2n_p=modparm(c2n_p_parm)
      c2n_z=modparm(c2n_z_parm)
      c2n_d=modparm(c2n_d_parm)
      balnutext=max(0.0,assimparm(balnutext_parm))
      balnutmin=min(1.0,max(0.0,assimparm(balnutmin_parm)))
      zfracb0=max(0.0,assimparm(zfracb0_parm))
      zfracb1=max(0.0,assimparm(zfracb1_parm))
      qrfmax=max(1.0,assimparm(qrfmax_parm))
      qafmax=max(1.0,assimparm(qafmax_parm))
      zrfmax=max(1.0,assimparm(zrfmax_parm))
      zafmax=max(1.0,assimparm(zafmax_parm))
      prfmax=max(1.0,assimparm(prfmax_parm))
      incphymin=(max(incphytiny,assimparm(incphymin_parm)))

! Calculate assimilation period in days

      assimperiod=ntstep_assim*timestep/86400.0

! Determine phytoplankton increment

      do i=1,imt
        do j=1,jmt
          incphy(i,j)=(cchl(i,j)/(mw_carbon*c2n_p))*incchl(i,j)
        enddo
      enddo

! Determine unadjusted nutrient balancing factor for surface layer 

      if (nbal_active.and.gl_active) then
        call calc_expected_nbf(imt,jmt,assimparm,assimperiod, &
     &                         avpgrow,avploss,avphy,incphy, &
     &                         expected_nbf,diag_active,diag)
      endif

      do i=1,imt
        do j=1,jmt

          if ((.not.nbal_active).or.(abs(incphy(i,j)).lt.incphymin)) then

            balnutsurf(i,j)=0.0

          else 

            if (gl_active.and.(expected_nbf(i,j).ge.0.0)) then
              balnutsurf(i,j)=expected_nbf(i,j)
            else 
              balnutsurf(i,j)=balnutext
            endif

          endif

        enddo
      enddo

! Determine deepest level for potential update 'kmax' (= deepest level 
! in water column)

      do i=1,imt
        do j=1,jmt
          k=1
          do while ((k.le.kmt(i,j)).and.(fm(i,j,k).gt.0.0))
            k=k+1
          enddo
          kmax(i,j)=k-1
	enddo
      enddo

! Determine deepest levels in current and maximum depth mixed layers.
! 'kmax' is the deepest level fully in the current mixed layer.
! 'kmldmax' is the deepest level fully or partly in the maximum depth mixed 
! layer.
! If sub-surface increments are to be suppressed then set 'kmax' to deepest
! level in current mixed layer. 

      do i=1,imt
        do j=1,jmt
          kmld(i,j)=0
          kmldmax(i,j)=0
          if (abs(incphy(i,j)).ge.incphymin) then
            do k=2,kmax(i,j)
              if ((kmld(i,j).eq.0).and.(zdz(i,j,k).gt.mld(i,j))) then
        	kmld(i,j)=k-1
              endif
              if ((kmldmax(i,j).eq.0).and.(zdz(i,j,k).ge.mldmax(i,j))) then
        	kmldmax(i,j)=k
              endif
            enddo
            if (kmld(i,j).eq.0) then
              kmld(i,j)=kmax(i,j)
            endif
            if (kmldmax(i,j).eq.0) then
              kmldmax(i,j)=kmax(i,j)
            endif
            if (.not.(subsurf_active)) then
              kmax(i,j)=kmld(i,j)
            endif
          endif
        enddo
      enddo

! Initialize diagnostics (excluding those specific to calc_expected_nbf)

      if (diag_active) then
        do i=1,imt
	  do j=1,jmt
            diag(i,j,ubalnut_d)=balnutsurf(i,j)
            diag(i,j,k_nlobound_d)=0
            diag(i,j,k_nhibound_d)=0
            diag(i,j,k_plobound_d)=0
            diag(i,j,k_zlobound_d)=0
            diag(i,j,k_zhibound_d)=0
            diag(i,j,k_dlobound_d)=0
            diag(i,j,kmld_d)=kmld(i,j)
            diag(i,j,kmldmax_d)=kmldmax(i,j)
            diag(i,j,kmax_d)=0
          enddo
	enddo
      endif

      if (diag_fulldepth_active) then
        do i=1,imt
	  do j=1,jmt
            do k=1,km
              diag_fulldepth(i,j,k,ubalphy_d)=0.0
              do m=1,n_inc_partial
        	diag_fulldepth(i,j,k,inc1_d+m-1)=0.0
        	diag_fulldepth(i,j,k,inc2_d+m-1)=0.0
              enddo
            enddo
	  enddo    
        enddo
      endif

! Calculate increments for water column 

      do i=1,imt
        do j=1,jmt

!         Initialize increment array

          do k=1,km
            do m=1,n_inc
              inc(i,j,k,m)=0.0
            enddo
          enddo

!         Initialize flags for indicating completion of water column processing. 
!         Secondary increments are only applied if the surface phytoplankton
!         increment is negative (implying a positive surface nutrient increment). 

          if (abs(incphy(i,j)).lt.incphymin) then
            done_phy=.true.
            done_nut=.true.
          else
            done_phy=.false.
            done_nut=((.not.nbal_active).or. &
       &              (.not.nutprof_active).or. &
       &              (incphy(i,j).gt.0.0)) 
          endif

          nodeepphyinc= &
       &    ((incphy(i,j).lt.0.0).and.(.not.deepneg_active)) &
       &    .or. &
       &    ((incphy(i,j).gt.0.0).and.(.not.deeppos_active))


!         Determine increments 

          k=1
          do while ((k.le.kmax(i,j)).and.(.not.(done_phy.and.done_nut)))

!           Initialize partial increments

            do m=1,n_inc_partial
              inc1(m)=0.0
              inc2(m)=0.0
            enddo

!           Initialize flag to indicate if 'p_specific_loss' called 
!           for current level

            ploss_valid=.false.

!           Initialize working variables for diagnostics 

            diag_ubalphy=0.0
            diag_nloflag=.false.
            diag_nhiflag=.false.
            diag_ploflag=.false.
            diag_zloflag=.false.
            diag_zhiflag=.false.
            diag_dloflag=.false.

!           Get background state 

            phy=max(0.0,bstate(i,j,k,phyto_tracer))
            nut=max(0.0,bstate(i,j,k,nutrient_tracer))
            zoo=max(0.0,bstate(i,j,k,zoo_tracer))
            det=max(0.0,bstate(i,j,k,dn_tracer))
            if (k.eq.1) then
              bphysurf=phy
            endif

!           Determine state-dependent analysis bounds imposed by restrictions 

            if (nbal_active) then

              if (((incphy(i,j).gt.0.0).and.(qrfmax.le.1.0)).or. &
       &          ((incphy(i,j).lt.0.0).and.(qafmax.le.1.0))) then
        	nutbound=nut
              else
        	q=nut/(grow_sat+nut)  
        	if (incphy(i,j).gt.0.0) then
                  qbound=q/qrfmax 
        	else
                  qbound=min(0.99999,q*qafmax)
        	endif
        	nutbound=qbound*grow_sat/(1.0-qbound)
              endif

              zoomin=zoo/zrfmax
              zoomax=zoo*zafmax

            endif

!           Primary increments 

            if (.not.done_phy) then

!             Set P increment

              if (k.eq.1) then
        	inc1(phy_t)=incphy(i,j)
              else
        	inc1(phy_t)=incphy(i,j)*(phy/bphysurf)
        	if (abs(inc1(phy_t)).gt.abs(inc(i,j,k-1,phyto_tracer))) &
       &        then
                  inc1(phy_t)=inc(i,j,k-1,phyto_tracer)
        	endif
              endif

              if (nbal_active) then

!               Determine balancing factors consistent with analysis bounds 

!               Note: zooplankton fraction of phytoplankton loss to the 
!               combined Z & D pools is calculated differently according to the 
!               sign of the Z & D increments.
!               Nutrient balancing factors greater than one imply Z (or D) 
!               increments have same sign as P increment. Otherwise sign is 
!               opposite.
!               Surface zooplankton loss fraction applies down to the maximum 
!               depth of mixing over the assimilation time step.

        	balnutmax=(nut-nutbound)/inc1(phy_t)
        	balzoomax_r=abs((zoo-zoomin)/inc1(phy_t))
        	balzoomax_a=abs((zoo-zoomax)/inc1(phy_t))
        	baldetmax=abs(det/inc1(phy_t))
        	balnut=balnutsurf(i,j)

        	if (balnut.gt.balnutmax) then
                  balnut=balnutmax
                  if (inc1(phy_t).gt.0.0) then
                    diag_nloflag=.true.
                  else
                    diag_nhiflag=.true.
                  endif
        	endif

        	if (((inc1(phy_t).lt.0.0).and.(balnut.gt.1.0)).or. &
       &            ((inc1(phy_t).gt.0.0).and.(balnut.lt.1.0))) then

!                 Negative Z & D increments - correcting for excessive loss so 
!                 zooplankton loss fraction is based on the model dynamics 

                  if (k.le.kmldmax(i,j)) then
                    phy_zf=avphy(i,j)
                  else
                    phy_zf=phy
                  endif
                  zfrac=min(1.0,max(0.0,zfracb0-zfracb1*phy_zf))

        	else

!                 Positive Z & D increments 
!                Insufficient loss correction applies   

                  if (k.le.kmldmax(i,j)) then
                    phy_zf=avphy(i,j) 
                    incphy_zf=incphy(i,j)
                  else
                    phy_zf=phy
                    incphy_zf=inc1(phy_t)
                  endif

                  zfrac=min(1.0,max(0.0, &
       &                zfracb0-zfracb1*(phy_zf+0.5*incphy_zf)))

        	endif

        	balzoo=(1.0-balnut)*zfrac

        	call adjusted_bal_p(inc1(phy_t),balnutmax, &
       &                            balzoomax_r,balzoomax_a,baldetmax, &
       &                            balnut,balzoo,baldet, &
       &                            diag_nloflag,diag_zloflag, &
       &                            diag_zhiflag,diag_dloflag)

!               Set balancing increments

        	inc1(nut_t)=-balnut*inc1(phy_t)
        	inc1(zoo_t)=-balzoo*inc1(phy_t)
        	inc1(det_t)=-baldet*inc1(phy_t)

              endif

            endif

!           Secondary increments - applied to each level while nutrient
!           is less than the analyzed nutrient for the level above.

            if ((k.gt.1).and.(.not.done_nut)) then

  !           Determine nutrient increment 

              nut=nut+inc1(nut_t)
!!MM Change minimum value to be > 0 to avoid zero divides later.
!!              inc2(nut_t)=max(0.0,min(anutabove-nut, &
!!       &                              inc(i,j,k-1,nut_t)-inc1(nut_t)))
              inc2(nut_t)=max(1.0e-6,min(anutabove-nut, &
       &                              inc(i,j,k-1,nut_t)-inc1(nut_t)))

              if (inc2(nut_t).gt.0.0) then

        	bphy=phy
        	phy=phy+inc1(phy_t)
        	zoo=zoo+inc1(zoo_t)
        	det=det+inc1(det_t)

!             Determine unadjusted phytoplankton balancing factor 

        	if (k.le.kmldmax(i,j)) then

!                 Use surface loss value 

                  balphy= &
       &            1.0/(1.0+0.5*assimperiod*(1-balnutmin)*avploss(i,j))

        	else

!                 Use local loss value 

                  call p_specific_loss &
       &                 (modparm,n2be_p,n2be_z,n2be_d, &
       &                  max(0.0,bstate(i,j,k,phyto_tracer)), &
       &                  max(0.0,bstate(i,j,k,zoo_tracer)), &
       &                  max(0.0,bstate(i,j,k,dn_tracer)), &
       &                  ploss,ploss_valid)

                  balphy=1.0/(1.0+0.5*assimperiod*(1-balnutmin)*ploss)

        	endif

        	diag_ubalphy=balphy

!               Set balancing factor limits.
!               Note that 'prfmax' applies only to the secondary
!               P increment. It is the permissible reduction factor
!               relative to the intermediate P.

        	balphymax=(phy-phy/prfmax)/inc2(nut_t)
        	balzoomax_r=(zoo-zoomin)/inc2(nut_t)
        	baldetmax=det/inc2(nut_t)

!               Determine balancing factors consistent with analysis bounds 
!               Note: Z & D increments are negative so the zooplankton loss
!               fraction is based on the model dynamics

        	if (balphy.gt.balphymax) then
                  balphy=balphymax
                  diag_ploflag=.true.
        	endif

        	if (k.le.kmldmax(i,j)) then
                  phy_zf=avphy(i,j)
        	else
                  phy_zf=bphy
        	endif

        	balzoo=(1.0-balphy)* &
       &          min(1.0,max(0.0,zfracb0-zfracb1*phy_zf))

        	call adjusted_bal_n(balphymax,balzoomax_r,baldetmax, &
       &                            balphy,balzoo,baldet, &
       &                            diag_ploflag,diag_zloflag, &
       &                            diag_dloflag)

!               Determine balancing increments

        	inc2(phy_t)=-balphy*inc2(nut_t)
        	inc2(zoo_t)=-balzoo*inc2(nut_t)
        	inc2(det_t)=-baldet*inc2(nut_t)

              endif
            endif

!           Nitrogen increments

            do m=1,n_inc_partial
              inc(i,j,k,m)=inc1(m)+inc2(m) 
            enddo

!           Carbonate system increments

            inc(i,j,k,dic_t)= &
       &      -c2n_p*inc(i,j,k,phy_t) &
       &      -c2n_z*inc(i,j,k,zoo_t) &
       &      -c2n_d*inc(i,j,k,det_t)

            inc(i,j,k,alk_t)=-inc(i,j,k,nut_t)

!           Set shallowest levels for constraints in diagnostics 

            if (diag_active.and.nbal_active) then
              if (diag_nloflag.and.(diag(i,j,k_nlobound_d).eq.0)) then
        	diag(i,j,k_nlobound_d)=k
              endif
              if (diag_nhiflag.and.(diag(i,j,k_nhibound_d).eq.0)) then
        	diag(i,j,k_nhibound_d)=k
              endif
              if (diag_ploflag.and.(diag(i,j,k_plobound_d).eq.0)) then
        	diag(i,j,k_plobound_d)=k
              endif
              if (diag_zloflag.and.(diag(i,j,k_zlobound_d).eq.0)) then
        	diag(i,j,k_zlobound_d)=k
              endif
              if (diag_zhiflag.and.(diag(i,j,k_zhibound_d).eq.0)) then
        	diag(i,j,k_zhibound_d)=k
              endif
              if (diag_dloflag.and.(diag(i,j,k_dlobound_d).eq.0)) then
        	diag(i,j,k_dlobound_d)=k
              endif
            endif

!           Determine level down to which increments apply
!           (1 level only below the mixed layer) 

            kend=max(kmld(i,j),k)

            do while (k.le.kend)

!             Copy surface increments if still in mixed layer

              if ((k.gt.1).and.(k.le.kmld(i,j))) then
        	do m=1,n_inc
                  inc(i,j,k,m)=inc(i,j,1,m)
        	enddo
              endif

!             Update diagnostics for level k

              if (diag_active) then
        	do m=1,n_inc
                  if (inc(i,j,k,m).ne.0.0) then
                    diag(i,j,kmax_d)=k
                  endif
        	enddo
              endif

              if (diag_fulldepth_active) then
        	diag_fulldepth(i,j,k,ubalphy_d)=diag_ubalphy
        	do m=1,n_inc_partial
                  diag_fulldepth(i,j,k,inc1_d+m-1)=inc1(m)
                  diag_fulldepth(i,j,k,inc2_d+m-1)=inc2(m)
        	enddo
              endif

              k=k+1

            enddo  ! Next level (without changing increments) 

!           Update completion flags:

!           Primary increments are applied down to the mixed layer depth 
!           maximum or below, according to control flags, 
!           while background P > 0.
!           In the special case of background P <= 0 at the surface, 
!           increments are applied within the current mixed layer only.         

            if (.not.done_phy) then
              done_phy=((k.gt.kmldmax(i,j).and.nodeepphyinc).or. &
       &                (bstate(i,j,k,phyto_tracer).le.0.0).or. &
       &                (bphysurf.le.0.0))
            endif

!           Secondary increments are applied to each level for which nutrient
!           is less than the analyzed nutrient for the level above, subject
!           to the restriction that nutrient increments cannot increase with
!           depth. Thus no further increments are possible when the increment 
!           above is zero.

            if (.not.done_nut) then
              done_nut=inc(i,j,k-1,nut_t).le.0.0
              if (.not.done_nut) then
        	anutabove=max(0.0,bstate(i,j,k-1,nutrient_tracer)) &
       &                  +inc(i,j,k-1,nut_t)
              endif
            endif

          enddo  ! Next level

        enddo  ! Next in row (j)
      enddo  ! Next in row (i)
      
!!!#else

!!!      write(6,*) 'subroutine bioanalysis' 
!!!      write(6,*) 'You should not have reached here'
!!!      write(6,*) 'Recompile with key_hadocc to use'

!!!#endif

      return
      end subroutine bio_analysis

!!!#if defined key_hadocc

! -------------------------------------------------------------------------
! CALC_EXPECTED_NBF
!
! Evaluates the expected value of the nutrient balancing function g(x,y) 
! over the joint p.d.f. p(x,y) given X + Y = -DP
!
! x is the growth component of the phytoplankton error 
!   drawn from the random variable X 
! y is the loss component of the phytoplankton error 
!   drawn from the random variable Y
! DP is the phytoplankton increment 'incphy' 
! (so -DP is an estimate of the net phytoplankton error)
!
! g(x,y) = u(x,y) + (1 - u(x,y)) Bmin + 0.5 dt (1 - Bmin) Bt(x,y)
!        = Bmin + (1 - Bmin) {u(x,y) + 0.5 dt Bt(x,y)}
!
! If x and y have the same sign: u(x,y) = x / (x + y) 
! Else if |x| > |y| : u(x,y) = 1
!      if |x| < |y| : u(x,y) = 0
!
! Bt(x,y) = min(Go - (x / (Po dt)), Lo + (y / (Po dt))) 
!
! Bmin is the fraction of phytoplankton loss to nutrient 'balnutmin'
! dt is the assimilation timestep 'assimperiod'
! Go is the model phytoplankton specific growth rate 'avpgrow' 
! Lo is the model phytoplankton specific loss rate 'avploss' 
! Po is the model phytoplankton concentration 'avphy'
!
! p(x,y) = p0(x) (r p1(y) + (1 - r) p2(y))
!
! r is the reliability (0-1) of the model phytoplankton specific loss rate 
! p0 is the p.d.f. for the growth error component X, given the bias-corrected 
!    growth rate estimate as the true growth rate
! p1 is the p.d.f. for the loss error component Y, given the primary 
!    bias-corrected loss rate estimate as the true loss rate (Y = Y1) 
! p2 is the p.d.f. for the loss error component Y, given the secondary 
!    bias-corrected loss rate estimate as the true loss rate (Y = Y2) 
!
! p0, p1 and p2 are log-normal  
! The corresponding normally distributed random variables are: 
!   Z0 = log(dt Po Go - X) 
!   Z1 = log(Y1 + dt Po Lo) 
!   Z2 = log(Y2 + dt Po Lo) 
! -------------------------------------------------------------------------

      subroutine calc_expected_nbf(imt,jmt,assimparm,assimperiod, &
     &                             avpgrow,avploss,avphy,incphy, &
     &                             expected_nbf,diag_active,diag) 

      implicit none

      real pi,incphytiny
      parameter (pi=3.1415927)
      parameter (incphytiny=1.0e-9)

! Assimilation scheme parameter indices

      integer balnutmin_parm,r_parm
      integer beta_g_parm,beta_l_parm,beta_m_parm
      integer a_g_parm,a_l_parm,a_m_parm,incphymin_parm
      integer integnstep_parm,pthresholdfrac_parm

      parameter (balnutmin_parm=3)
      parameter (r_parm=4)
      parameter (beta_g_parm=5)
      parameter (beta_l_parm=6)
      parameter (beta_m_parm=7)
      parameter (a_g_parm=8)
      parameter (a_l_parm=9)
      parameter (a_m_parm=10)
      parameter (incphymin_parm=18)
      parameter (integnstep_parm=19)
      parameter (pthresholdfrac_parm=20)

! Diagnostic indices

      integer xmin_d,xmax_d,pinteg_d,expected_x_d,expected_y_d, &
     &        expected_u_d,expected_bt_d 
      parameter (xmin_d=11)
      parameter (xmax_d=12)
      parameter (pinteg_d=13)
      parameter (expected_x_d=14)
      parameter (expected_y_d=15)
      parameter (expected_u_d=16)
      parameter (expected_bt_d=17)

! Arguments

      integer imt                 ! IN Number of points in horizontal
      integer jmt                 ! IN Number of points in vertical
      real assimparm(*)           ! IN Assimilation scheme parameters
      real assimperiod            ! IN Assimilation timestep (d) 
      real avpgrow(imt,jmt)       ! IN Model growth rate for assim. timestep (/d)
      real avploss(imt,jmt)       ! IN Model loss rate for assim. timestep (/d)
      real avphy(imt,jmt)         ! IN Model P for assimilation timestep (mmolN/m3)
      real incphy(imt,jmt)        ! IN Phytoplankton increment DP (mmolN/m3)
      real expected_nbf(imt,jmt)  ! OUT Expected value of g(x,y)
      logical diag_active         ! IN Switch for diagnostics 
      real diag(imt,jmt,*)        ! OUT Diagnostics

! Local variables

!     External assimilation scheme parameter values

      real balnutmin ! Fraction of phytoplankton loss to nutrient
      real r         ! Reliability of model specific growth rate
      real beta_g    ! Low rate bias correction for growth rate estimator (/d) 
      real beta_l    ! Low rate bias correction for primary loss rate 
                     ! estimator (/d) 
      real beta_m    ! Low rate bias correction for secondary loss rate 
                     ! estimator (/d) 
      real a_g       ! Error s.d. for log10 of growth rate estimator
      real a_l       ! Error s.d. for log10 of primary loss rate estimator
      real a_m       ! Error s.d. for log10 of secondary loss rate estimator
      real incphymin ! Min. size of non-zero phytoplankton increment (mmolN/m3)
      integer integnstep  
                     ! Number of steps for p.d.f. integral evaluation
      real pthresholdfrac
                     ! Fractional threshold level for setting p.d.f.
                     ! integration bounds (=fraction of p.d.f. value at ref. 
                     ! point below which p.d.f. is taken to be zero)
      

!     Working variables

      real gstar              ! Growth rate estimate 
      real lstar              ! Primary loss rate estimate
      real mstar              ! Secondary loss rate estimate
      real m0                 ! Z0 mean 
      real m1                 ! Z1 mean 
      real m2                 ! Z2 mean 
      real s0                 ! Z0 s.d.
      real s1                 ! Z1 s.d.
      real s2                 ! Z2 s.d.
      real pdt                ! Model phytoplankton * assimilation timestep
      real xmaxpdf            ! Max. x for which joint p.d.f. is non-zero 
      real yminpdf            ! Min. y for which joint p.d.f. is non-zero
      real xmaxline           ! Maximum x on the line y = -(x + DP) for which
                              ! joint p.d.f. is non-zero
      real xmin               ! Lower bound used for integrating p.d.f. 
      real xmax               ! Upper bound used for integrating p.d.f.
      real logpinteg          ! Log of p.d.f. integral along y = -(x + DP) 
                              ! Note: dy * exp(logpinteg) approximates
                              !     Prob(DP - dy/2 < -(Y + X) < DP + dy/2)
                              ! for small dy
      real expected_x         ! Expected x given Y = -(X + DP) (*)
                              ! (or fixed x if no uncertainty in growth rate)
      real expected_y         ! Expected y given Y = -(X + DP) (*)
                              ! (or fixed y if no uncertainty in loss rate)
      real expected_u         ! Expected u(x,y) given Y = -(X + DP) (*) 
      real expected_bt        ! Expected Bt(x,y) given Y = -(X + DP) (*) 
                              ! (*) If x or y are fixed and it is impossible 
                              ! to satisfy the condition Y = -(X + DP), we then 
                              ! satisy Y = -(X + DP) + dy where |dy| is as small
                              ! as possible for non-zero p(x,y) 
                              ! (If x and y are fixed the condition is ignored) 

      real var0,var1,var2
      real a
      real b0,b1,b2
      real k0,k1,k2
      real loss
      integer i,j
      logical fixed_growth
      logical fixed_loss
      logical fixed_l
      logical fixed_m
      logical valid_flag
      logical pinteg_flag

! Local functions

!      real scaled_pdf
!      real pintegral

! Get parameter values

      balnutmin=min(1.0,max(0.0,assimparm(balnutmin_parm)))
      r=min(1.0,max(0.0,assimparm(r_parm)))
      beta_g=max(0.0,assimparm(beta_g_parm))
      beta_l=max(0.0,assimparm(beta_l_parm))
      beta_m=max(0.0,assimparm(beta_m_parm))
      a=log10(exp(1.0))
      s0=max(0.0,assimparm(a_g_parm))/a
      s1=max(0.0,assimparm(a_l_parm))/a
      s2=max(0.0,assimparm(a_m_parm))/a
      incphymin=(max(incphytiny,assimparm(incphymin_parm)))
      integnstep=max(2,int(assimparm(integnstep_parm)+0.5))
      pthresholdfrac=min(1.0,max(0.0,assimparm(pthresholdfrac_parm)))

      var0=s0*s0
      var1=s1*s1
      var2=s2*s2
      a=1/sqrt(2*pi)
      if (s0.ne.0.0) then 
        k0=a/s0
        b0=-0.5/var0
      endif
      if (s1.ne.0.0) then 
        k1=r*a/s1
        b1=-0.5/var1
      endif
      if (s2.ne.0.0) then 
        k2=(1.0-r)*a/s2
        b2=-0.5/var2
      endif

! Process row

      do i=1,imt
        do j=1,jmt

          if (abs(incphy(i,j)).lt.incphymin) then

            expected_nbf(i,j)=0.0

          else

!           Calculate bias-corrected growth and loss rate estimates
!           and scaling factor for conversion to phytoplankton units 

            if (avpgrow(i,j).lt.2.0*beta_g) then
              gstar=beta_g+avpgrow(i,j)**2/(4.0*beta_g)
            else
              gstar=avpgrow(i,j)
            endif

            if (avploss(i,j).lt.2.0*beta_l) then
              lstar=beta_l+avploss(i,j)**2/(4.0*beta_l)
            else
              lstar=avploss(i,j)
            endif

            if (avpgrow(i,j).lt.2.0*beta_m) then
              mstar=beta_m+avpgrow(i,j)**2/(4.0*beta_m)
            else
              mstar=avpgrow(i,j)
            endif

            pdt=max(1.0e-6,avphy(i,j)*assimperiod)

!           Check for uncertainty in growth and loss estimates

            fixed_growth=((s0.le.0.0).or.(gstar.le.0.0)) 
            fixed_l=((s1.le.0.0).or.(lstar.le.0.0)) 
            fixed_m=((s2.le.0.0).or.(mstar.le.0.0)) 

            if (fixed_growth) then
              expected_x=pdt*(avpgrow(i,j)-gstar)
            endif

            if (fixed_l.and.(r.ge.1.0)) then
              fixed_loss=.TRUE.
              loss=lstar
              expected_y=pdt*(avploss(i,j)-lstar)
            else if (fixed_m.and.(r.le.0.0)) then
              fixed_loss=.TRUE.
              loss=mstar
              expected_y=pdt*(avpgrow(i,j)-mstar)
            else
              fixed_loss=.FALSE.
            endif

            valid_flag=.TRUE.
            pinteg_flag=.FALSE. 

!           Determine upper p.d.f. limits

            xmaxpdf=pdt*avpgrow(i,j)
            yminpdf=-pdt*avploss(i,j)
            xmaxline=min(xmaxpdf,-yminpdf-incphy(i,j))

!           Calculate expected values of u and Bt

            if (fixed_l.and.fixed_m.and.(r.gt.0.0).and.(r.lt.1.0)) then

!             Primary and secondary loss rate estimates are incompatible

              valid_flag=.FALSE. 

            else if (fixed_growth.and.fixed_loss) then

!             No uncertainty in growth or loss estimates 
!             so evaluate u(x,y) and Bt(x,y) ignoring phytoplankton increment 

              if (expected_x+expected_y.eq.0.0) then
        	expected_u=0.5
              else
        	expected_u= &
       &          min(max(0.0,expected_x/(expected_x+expected_y)),1.0)
              endif
              expected_bt=min(gstar,loss)

            else if (fixed_growth) then

!             Uncertainty in loss estimate only. 
!             Evaluate u(x,y) and Bt(x,y) for fixed x on the line 
!             y = -(x + DP) (or as close to the line as possible given the 
!             minimum y set by the model loss rate).

              expected_y=max(-expected_x-incphy(i,j),yminpdf)
              if (expected_x+expected_y.eq.0.0) then
        	expected_u=0.5
              else
        	expected_u= &
       &          min(max(0.0,expected_x/(expected_x+expected_y)),1.0)
              endif 
              expected_bt=max((xmaxline-expected_x),0.0)/pdt

            else if (fixed_loss) then

!             Uncertainty in growth estimate only. 
!             Evaluate u(x,y) and Bt(x,y) for fixed y on the line 
!             y = -(x + DP) (or as close to the line as possible given the
!             maximum x set by the model growth rate).

              expected_x=min(-expected_y-incphy(i,j),xmaxpdf)
              if (expected_x+expected_y.eq.0.0) then
        	expected_u=0.5
              else
        	expected_u= &
       &          min(max(0.0,expected_x/(expected_x+expected_y)),1.0)
              endif
              expected_bt=(xmaxline-expected_x)/pdt

            else

!             Uncertainty in growth and loss estimates. 
!             Need to find weighted averages of u(x,y) and Bt(x,y) along 
!             the line y = -(x + DP). Call 'process_pdf' to do the integration.

              m0=log(pdt*gstar)-0.5*var0
              m1=log(pdt*lstar)-0.5*var1
              m2=log(pdt*mstar)-0.5*var2

              call process_pdf(integnstep,pthresholdfrac, &
       &                       xmaxpdf,yminpdf,m0,m1,m2,k0,k1,k2, &
       &                       b0,b1,b2,incphy(i,j),xmaxline, &
       &                       xmin,xmax,logpinteg, &
       &                       expected_u,expected_x,valid_flag)

              expected_y=-expected_x-incphy(i,j)
              expected_bt=(xmaxline-expected_x)/pdt

              pinteg_flag=valid_flag

            endif

!           Calculate expected value of nutrient balancing function g(x,y)

            if (valid_flag) then
              expected_nbf(i,j)=balnutmin+(1.0-balnutmin)* &
       &                      (min(max(0.0,expected_u),1.0)+ &
       &                       0.5*assimperiod*expected_bt)
            else
              expected_x=0.0
              expected_y=0.0
              expected_u=0.0
              expected_bt=0.0
              expected_nbf(i,j)=-1.0
            endif

          endif

!         Diagnostics

          if (diag_active) then
            if (abs(incphy(i,j)).lt.incphymin) then
              diag(i,j,xmin_d)=0.0
              diag(i,j,xmax_d)=0.0
              diag(i,j,pinteg_d)=0.0
              diag(i,j,expected_x_d)=0.0
              diag(i,j,expected_y_d)=0.0
              diag(i,j,expected_u_d)=0.0
              diag(i,j,expected_bt_d)=0.0
            else
              if (pinteg_flag) then
        	diag(i,j,xmin_d)=xmin
        	diag(i,j,xmax_d)=xmax
        	diag(i,j,pinteg_d)=exp(logpinteg)
              else
        	diag(i,j,xmin_d)=0.0
        	diag(i,j,xmax_d)=0.0
        	diag(i,j,pinteg_d)=0.0
              endif
              diag(i,j,expected_x_d)=expected_x
              diag(i,j,expected_y_d)=expected_y
              diag(i,j,expected_u_d)=expected_u
              diag(i,j,expected_bt_d)=expected_bt
            endif
          endif

        enddo
      enddo

      return
      end subroutine calc_expected_nbf

! -------------------------------------------------------------------------
! PROCESS_PDF
!
! Calculate the expected values of u(x,y) and x given Y = -(X + DP).
! This subroutine also reports the finite integration bounds used ('xmin' & 
! 'xmax') and the log of the unscaled p.d.f. integral along the line 
! y = -(x + DP) ('logpinteg')
! -------------------------------------------------------------------------

      subroutine process_pdf(nstep,pthresholdfrac,xmaxpdf,yminpdf, &
     &                       m0,m1,m2,k0,k1,k2,b0,b1,b2,incphy, &
     &                       xmaxline,xmin,xmax,logpinteg,expected_u, &
     &                       expected_x,valid)

      implicit none

      integer nstep
      real pthresholdfrac 
      real xmaxpdf        
      real yminpdf       
      real m0,m1,m2
      real b0,b1,b2
      real k0,k1,k2
      real incphy
      real xmaxline     
      real xmin
      real xmax
      real logpinteg  
      real expected_u    
      real expected_x  
      logical valid

!     Working variables

      real scaling            ! Exponent for scaling used to avoid underflow
      real xref               ! x co-ordinate of scaling reference point

      real scaled_pthreshold
      real scaling1,scaling2
      real xref1,xref2
      logical nointeg1,nointeg2
      real xmin1,xmin2
      real xmax1,xmax2
      logical xfactor
      real ixp
      real ip
      integer i

! Local functions

!      real scaled_pdf
!      real pintegral

!     Determine scaling factor for avoiding underflow based on p.d.f.
!     value at a suitable reference point on the line y = -(x + DP).        
!     The two joint p.d.f. components r * p0 * p1 and (1 - r) * p0 * p2
!     are examined separately.  
!     The scaling used is that for whichever p.d.f. component is 
!     larger at its chosen reference point. 

      if (k2.le.0.0) then

        call get_scaling_exponent(xmaxpdf,yminpdf,xmaxline, &
     &                            m0,m1,k0,k1,b0,b1,incphy, &
     &                            xref,scaling)

      else if (k1.le.0.0) then

        call get_scaling_exponent(xmaxpdf,yminpdf,xmaxline, &
     &                            m0,m2,k0,k2,b0,b2,incphy, &
     &                            xref,scaling)

      else

        call get_scaling_exponent(xmaxpdf,yminpdf,xmaxline, &
     &                            m0,m1,k0,k1,b0,b1,incphy, &
     &                            xref1,scaling1)

        call get_scaling_exponent(xmaxpdf,yminpdf,xmaxline, &
     &                            m0,m2,k0,k2,b0,b2,incphy, &
     &                            xref2,scaling2)

        if (scaling1.le.scaling2) then
          scaling=scaling1
          xref=xref1
        else
          scaling=scaling2
          xref=xref2
        endif

      endif

!     Determine minimum and maximum x bounds for integration such that
!     all p.d.f. values above a threshold are included.
!     The threshold is set at a fraction 'pthresholdfrac' of the p.d.f.
!     value at the scaling reference point.
!     Bounds are determined for each of the two joint p.d.f. components 
!     separately such that all p.d.f. component values above half of the 
!     threshold value are included. 

!     Calculate threshold

      if (pthresholdfrac.le.0.0) then
        valid=.FALSE.
        return
      endif

      scaled_pthreshold= &
     &  0.5*pthresholdfrac*scaled_pdf(xref,-(xref+incphy), &
     &                                xmaxpdf,yminpdf, &
     &                                m0,m1,m2,k0,k1,k2,b0,b1,b2, &
     &                                scaling)

!     Get bounds for p.d.f. component r * p0 * p1 
 
      call get_integration_bounds(xmaxpdf,yminpdf,m0,m1,k0,k1, &
     &                            b0,b1,scaling,scaled_pthreshold, &
     &                            incphy,nointeg1,xmin1,xmax1)

!     Get bounds for p.d.f. component (1 - r) * p0 * p2

      call get_integration_bounds(xmaxpdf,yminpdf,m0,m2,k0,k2, &
     &                            b0,b2,scaling,scaled_pthreshold, &
     &                            incphy,nointeg2,xmin2,xmax2)

!     Select outermost bounds

      if (nointeg1) then
        xmin=xmin2
        xmax=xmax2
      else if (nointeg2) then
        xmin=xmin1
        xmax=xmax1
      else
        xmin=min(xmin1,xmin2)
        xmax=max(xmax1,xmax2)
      endif

!     Calculate scaled p.d.f integral along y = -(x + DP) 

      xfactor=.FALSE.
      ip=pintegral(xfactor,xmin,xmax, &
     &             incphy,xmaxpdf,yminpdf,m0,m1,m2, &
     &             k0,k1,k2,b0,b1,b2,scaling,nstep)

      if (ip.le.0.0) then

        valid=.FALSE. 

      else

        logpinteg=log(ip)-scaling

!       Calculate expected u(x,y), given Y = -(X + DP) 

        if (incphy.lt.0.0) then

          xfactor=.TRUE.
          ixp=pintegral(xfactor,max(0.0,xmin),min(-incphy,xmax), &
     &                  incphy,xmaxpdf,yminpdf,m0,m1,m2, &
     &                  k0,k1,k2,b0,b1,b2,-logpinteg,nstep)

          xfactor=.FALSE.
          ip=pintegral(xfactor,max(-incphy,xmin),xmax, &
     &                 incphy,xmaxpdf,yminpdf,m0,m1,m2, &
     &                 k0,k1,k2,b0,b1,b2,-logpinteg,nstep)

          expected_u=-ixp/incphy+ip

        else

          xfactor=.FALSE.
          ip=pintegral(xfactor,xmin,min(-incphy,xmax), &
     &                 incphy,xmaxpdf,yminpdf,m0,m1,m2, &
     &                 k0,k1,k2,b0,b1,b2,-logpinteg,nstep)
 
          xfactor=.TRUE.
          ixp=pintegral(xfactor,max(-incphy,xmin),min(0.0,xmax), &
     &                  incphy,xmaxpdf,yminpdf,m0,m1,m2, &
     &                  k0,k1,k2,b0,b1,b2,-logpinteg,nstep)

          expected_u=ip-ixp/incphy

        endif

!       Calculate expected x, given Y = -(X + DP) 

        xfactor=.TRUE.
        expected_x=pintegral(xfactor,xmin,xmax, &
     &                       incphy,xmaxpdf,yminpdf,m0,m1,m2, &
     &                       k0,k1,k2,b0,b1,b2,-logpinteg,nstep)

        valid=.TRUE.

      endif

      return
      end subroutine process_pdf

! -------------------------------------------------------------------------
! GET_SCALING_EXPONENT
!
! Subroutine to determine a suitable scaling factor to avoid underflow when
! integrating the 2-D log-normal function along the line y = -(x + DP).
! The scaling is based on the greater of the two function values at the
! points where the line crosses axes passing through the function maximum.
! The exponent for the scaling factor is returned, together with the
! x co-ordinate of the chosen reference point.
! -------------------------------------------------------------------------

      subroutine get_scaling_exponent(xmaxpdf,yminpdf,xmaxline, &
     &                                mx,my,kx,ky,bx,by,incphy, &
     &                                xref,scaling_exponent)

      implicit none

      real amp
      parameter (amp=30)      ! Exponent of scaled function at ref. point

      real xmaxpdf
      real yminpdf
      real xmaxline
      real mx
      real my
      real kx
      real ky
      real bx
      real by
      real incphy 
      real xref
      real scaling_exponent

      real xc,yc
      real xyc
      real x1,x2
      real zx,zy
      real t1,t2

!     Find abscissae of axis crossing points for axes with origin at
!     function maximum. These define potential reference points.
!     Use xmaxline instead if less.

      xc=xmaxpdf-exp(mx+1.0/(2.0*bx))
      yc=yminpdf+exp(my+1.0/(2.0*by))
      xyc=-yc-incphy
      if (xyc.le.xc) then
        x1=xyc
        x2=min(xc,xmaxline)
      else
        x1=xc
        x2=min(xyc,xmaxline)
      endif

!     Determine scaling exponent from function value at 'x1' or 'x2'
!     (whichever is greater)
!     N.B. function may go to zero at 'x2' so allow for this first

!     't1' and 't2' are the p.d.f. exponents at 'x1' and 'x2'

      zx=log(xmaxpdf-x1)
      zy=log(-x1-incphy-yminpdf)
      t1=bx*(zx-mx)**2-zx+by*(zy-my)**2-zy
      if (x2.eq.xmaxline) then
        scaling_exponent=log(1.0/(kx*ky))-t1+amp
        xref=x1
      else
        zx=log(xmaxpdf-x2)
        zy=log(-x2-incphy-yminpdf)
        t2=bx*(zx-mx)**2-zx+by*(zy-my)**2-zy
        if (t1.ge.t2) then
          scaling_exponent=log(1.0/(kx*ky))-t1+amp
          xref=x1
        else
          scaling_exponent=log(1.0/(kx*ky))-t2+amp
          xref=x2
        endif
      endif

      return
      end subroutine get_scaling_exponent

! -------------------------------------------------------------------------
! GET_INTEGRATION_BOUNDS
!
! Routine to determine integration bounds on the line y = -(x + DP) which 
! ensure that all of the 2-D log-normal function values above a required 
! threshold are included. 
!
! First, the smallest box containing the threshold contour 
! is found. (The box touches this contour where the contour intersects 
! axes passing through the function maximum.) 
! x limits are then determined for the line where it intersects the box. 
! (N.B. This is simpler and quicker than finding the points where the line 
! intersects the threshold contour itself.)
! This routine works with the scaled function to avoid underflow. 
! -------------------------------------------------------------------------

      subroutine get_integration_bounds(xmaxpdf,yminpdf, &
     &                                  mx,my,kx,ky,bx,by, &
     &                                  scaling,scaled_pthreshold, &
     &                                  incphy,nointeg,xmin,xmax)

      implicit none

      real xmaxpdf
      real yminpdf
      real mx
      real my
      real kx
      real ky
      real bx
      real by
      real scaling
      real scaled_pthreshold
      real incphy
      logical nointeg
      real xmin
      real xmax

      real c0,c1
      real rootsq,root
      real ymin
      real ymax

!     Check if peak is below threshold 
!     (Checking in y direction)

      if (ky.le.0.0) then
        xmin=0.0
        xmax=0.0
        nointeg=.TRUE.
        return
      endif

      c0=mx+my+log(scaled_pthreshold/(kx*ky))-scaling
      rootsq=by*(4.0*c0+1.0/bx)+1.0
      if (rootsq.le.0.0) then
        xmin=0.0
        xmax=0.0
        nointeg=.TRUE.
        return
      endif

!     Find box
          
      root=sqrt(rootsq)
      c1=2.0*by*my+1.0
      ymin=yminpdf+exp((c1+root)*0.5/by)
      ymax=yminpdf+exp((c1-root)*0.5/by)

      root=sqrt(max(bx*(4.0*c0+1.0/by)+1.0,0.0))
      c1=2.0*bx*mx+1.0
      xmin=xmaxpdf-exp((c1-root)*0.5/bx)
      xmax=xmaxpdf-exp((c1+root)*0.5/bx)

!     Find intersections of line with box 

      xmin=max(xmin,-ymax-incphy)
      xmax=min(xmax,-ymin-incphy)

      nointeg=.FALSE.
      return
      end subroutine get_integration_bounds

! -------------------------------------------------------------------------
! PINTEGRAL
!
! Function to integrate the scaled joint p.d.f. p (or x * p) along the line 
! y = -(x + DP) between finite bounds 'xlo' and 'xhi'. 
! The integral is evaluated numerically using Simpson's rule. 
! -------------------------------------------------------------------------

      real function pintegral(xfactor_on,xlo,xhi,incphy, &
     &                        xmaxpdf,yminpdf,m0,m1,m2,k0,k1,k2, &
     &                        b0,b1,b2,scaling,nstep)

      implicit none

      logical xfactor_on
      real xlo
      real xhi
      real incphy
      real xmaxpdf
      real yminpdf
      real m0
      real m1
      real m2
      real k0
      real k1 
      real k2 
      real b0
      real b1 
      real b2 
      real scaling
      integer nstep

      real x,xmid
      real xstep
      real plo,phi,p,pmid
      real sum
      integer i

!      real scaled_pdf

      if (xlo.ge.xhi) then
        pintegral=0.0
        return
      endif

      xstep=(xhi-xlo)/nstep

      xmid=xlo+0.5*xstep
      plo=scaled_pdf(xlo,-(xlo+incphy),xmaxpdf,yminpdf,m0,m1,m2, &
     &               k0,k1,k2,b0,b1,b2,scaling)
      phi=scaled_pdf(xhi,-(xhi+incphy),xmaxpdf,yminpdf,m0,m1,m2, &
     &               k0,k1,k2,b0,b1,b2,scaling)
      pmid=scaled_pdf(xmid,-(xmid+incphy),xmaxpdf,yminpdf,m0,m1,m2, &
     &                k0,k1,k2,b0,b1,b2,scaling)

      if (xfactor_on) then 
        sum=0.5*(xlo*plo+xhi*phi)+2*xmid*pmid
      else
        sum=0.5*(plo+phi)+2*pmid
      endif

      do i=1,nstep-1

        x=xlo+i*xstep

        xmid=x+0.5*xstep
        p=scaled_pdf(x,-(x+incphy),xmaxpdf,yminpdf,m0,m1,m2, &
     &               k0,k1,k2,b0,b1,b2,scaling)
        pmid=scaled_pdf(xmid,-(xmid+incphy),xmaxpdf,yminpdf,m0,m1,m2, &
     &                  k0,k1,k2,b0,b1,b2,scaling)

        if (xfactor_on) then 
          sum=sum+x*p+2*xmid*pmid
        else
          sum=sum+p+2*pmid
        endif

      enddo

      pintegral=sum*xstep/3.0

      return
      end function pintegral

! -------------------------------------------------------------------------
! SCALED_PDF
!
! Function returning the scaled joint probability density function at the
! point (x,y) i.e. p(x,y) * exp(scaling).
! The scaling is applied to avoid large negative exponents causing 
! underflow.
! -------------------------------------------------------------------------

      real function scaled_pdf(x,y,xmaxpdf,yminpdf, &
     &                         m0,m1,m2,k0,k1,k2,b0,b1,b2,scaling)

      implicit none

      real x 
      real y 
      real xmaxpdf
      real yminpdf
      real m0
      real m1
      real m2
      real k0
      real k1
      real k2
      real b0
      real b1
      real b2
      real scaling

      real zx,zy 
      real c

      if ((x.ge.xmaxpdf).or.(y.le.yminpdf)) then
        scaled_pdf=0.0
        return
      endif

      zx=log(xmaxpdf-x)
      zy=log(y-yminpdf)
      c=b0*(zx-m0)**2-zx-zy+scaling

      scaled_pdf=k0*(k1*exp(c+b1*(zy-m1)**2)+k2*exp(c+b2*(zy-m2)**2))

      return
      end function scaled_pdf

! -------------------------------------------------------------------------
! ADJUSTED_BAL_P
! 
! Routine to determine adjusted balancing factors for a given phytoplankton
! increment as a function of the increment, preliminary nutrient and 
! zooplankton balancing factors and the state-dependent limits.
! The input nutrient balancing factor is assumed to satisfy the nutrient
! restriction.
!
! (1) Adjust zooplankton factor if necessary 
! (2) Determine detritus factor by conservation and adjust if necessary
! (3) Re-determine zooplankton factor by conservation and adjust if 
!     necessary (This contains the adjustment within the combined 
!     zooplankton and detritus pools if possible)
! (4) Re-determine nutrient factor by conservation
!     (This step only changes the nutrient factor if there is
!     insufficient detritus for the adjustment to be contained within
!     Z & D)
!
! N.B. It is not always possible to fully conserve nitrogen for a positive 
! phytoplankton increment.
! -------------------------------------------------------------------------

      subroutine adjusted_bal_p(incphy,balnutmax, &
     &                          balzoomax_r,balzoomax_a,baldetmax, &
     &                          balnut,balzoo,baldet, &
     &                          diag_nflag,diag_zrflag,diag_zaflag, &
     &                          diag_dflag)

      implicit none

      real incphy
      real balnutmax
      real balzoomax_r
      real balzoomax_a
      real baldetmax
      real balnut        
      real balzoo     
      real baldet    
      logical diag_nflag
      logical diag_zrflag
      logical diag_zaflag
      logical diag_dflag

      if (incphy.lt.0.0) then

!       -ve P increment so Z (or D) increment has the same sign as 
!       Z (or D) balancing factor: 
!       -ve Z (or D) factors imply decrease, +ve factors imply increase

!       Adjustment to 'balnut' only occurs when there is insufficient D 
!       which implies 'baldet' & 'balzoo' are -ve. Reducing the
!       size of 'baldet' and/or 'balzoo' therefore reduces 'balnut'.
!       Adjusted 'balnut cannot therefore exceed the previously imposed
!       limit 'balnutmax' so nitrogen is always conserved.

        if (balzoo.lt.-balzoomax_r) then
          balzoo=-balzoomax_r
          diag_zrflag=.true.
        else if (balzoo.gt.balzoomax_a) then
          balzoo=balzoomax_a
          diag_zaflag=.true.
        endif

        baldet=1-balnut-balzoo
        if (baldet.lt.-baldetmax) then
          baldet=-baldetmax
          diag_dflag=.true.
        endif

        balzoo=1-balnut-baldet
        if (balzoo.lt.-balzoomax_r) then
          balzoo=-balzoomax_r
          diag_zrflag=.true.
        else if (balzoo.gt.balzoomax_a) then
          balzoo=balzoomax_a
          diag_zaflag=.true.
        endif

        balnut=1-balzoo-baldet

      else

!       +ve P increment so Z (or D) increment has opposite sign to 
!       Z (or D) balancing factor: 
!       -ve Z (or D) factors imply increase, +ve factors imply decrease

!       Adjustment to 'balnut' only occurs when there is insufficient D 
!       which implies 'baldet' & 'balzoo' are +ve. Reducing the
!       size of 'baldet' and/or 'balzoo' therefore increases 'balnut'
!       potentially above the previously imposed limit 'balnutmax'.
!       Any excess increase is suppressed. In this case, nitrogen is not 
!       conserved by the assimilation scheme: nitrogen is created 
!       to satisfy the required phytoplankton increment.

        if (balzoo.gt.balzoomax_r) then
          balzoo=balzoomax_r
          diag_zrflag=.true.
        else if (balzoo.lt.-balzoomax_a) then
          balzoo=-balzoomax_a
          diag_zaflag=.true.
        endif

        baldet=1-balnut-balzoo
        if (baldet.gt.baldetmax) then
          baldet=baldetmax
          diag_dflag=.true.
        endif

        balzoo=1-balnut-baldet
        if (balzoo.gt.balzoomax_r) then
          balzoo=balzoomax_r
          diag_zrflag=.true.
        else if (balzoo.lt.-balzoomax_a) then
          balzoo=-balzoomax_a
          diag_zaflag=.true.
        endif

        balnut=1-balzoo-baldet
        if (balnut.gt.balnutmax) then
          balnut=balnutmax
          diag_nflag=.true.
        endif

      endif

      return
      end subroutine adjusted_bal_p

! -------------------------------------------------------------------------
! ADJUSTED_BAL_N
! 
! Routine to determine adjusted balancing factors for a given nutrient 
! increment as a function of the increment, preliminary phytoplankton and 
! zooplankton balancing factors and the state-dependent limits.
! The input phytoplankton balancing factor is assumed to satisfy the 
! phytoplankton restriction.
!
! (1) Adjust zooplankton factor if necessary. 
! (2) Determine detritus factor by conservation and adjust if necessary
! (3) Re-determine zooplankton factor by conservation and adjust if
!     necessary (This contains the adjustment within the combined
!     zooplankton and detritus pools if possible)
! (4) Re-determine phytoplankton factor by conservation
!     (This step only changes the phytoplankton factor if there is
!     insufficient detritus for the adjustment to be contained within
!     Z & D)
!
! N.B. It is not always possible to fully conserve nitrogen.
! Conservation may imply increasing 'balphy' above the limit 'balphymax'.
! Any excess increase is suppressed. In this case, there is insufficient
! nitrogen present to satisfy the required nutrient increment and nitrogen  
! must be created. 
! -------------------------------------------------------------------------

      subroutine adjusted_bal_n(balphymax,balzoomax,baldetmax, &
     &                          balphy,balzoo,baldet, &
     &                          diag_pflag,diag_zflag,diag_dflag)

      implicit none

      real balphymax
      real balzoomax
      real baldetmax
      real balphy        
      real balzoo     
      real baldet    
      logical diag_pflag
      logical diag_zflag
      logical diag_dflag

      if (balzoo.gt.balzoomax) then
        balzoo=balzoomax
        diag_zflag=.true.
      endif

      baldet=1-balphy-balzoo
      if (baldet.gt.baldetmax) then
        baldet=baldetmax
        diag_dflag=.true.
      endif

      balzoo=1-balphy-baldet
      if (balzoo.gt.balzoomax) then
        balzoo=balzoomax
        diag_zflag=.true.
      endif

      balphy=1-balzoo-baldet
      if (balphy.gt.balphymax) then
        balphy=balphymax
        diag_pflag=.true.
      endif

      return
      end subroutine adjusted_bal_n

! -------------------------------------------------------------------------
!
! P_SPECIFIC_LOSS
!
! Routine to calculate phytoplankton specific loss rate (/d) as defined by 
! the HadOCC model in terms of phytoplankton, zooplankton and detritus 
! concentrations. 
! Losses are due to grazing, mortality and respiration.
! -------------------------------------------------------------------------
  

      subroutine p_specific_loss(parm,n2be_p,n2be_z,n2be_d,p,z,dn, &
     &                           specific_loss,valid)

      implicit none

! Model parameter indices

      integer resp_rate_parm,pmort_rate_parm,phyto_min_parm
      integer graze_threshold_parm,holling_coef_parm
      integer graze_sat_parm,graze_max_parm

      parameter (resp_rate_parm=5)
      parameter (pmort_rate_parm=6)
      parameter (phyto_min_parm=7)
      parameter (graze_threshold_parm=13)
      parameter (holling_coef_parm=14)
      parameter (graze_sat_parm=15)
      parameter (graze_max_parm=16)

! Arguments

      real parm(*)             ! IN Model parameters   
      real n2be_p              ! IN Phytoplankton biomass equivalent factor 
      real n2be_z              ! IN Zooplankton biomass equivalent factor
      real n2be_d              ! IN Detritus biomass equivalent factor 
      real p                   ! IN Phytoplankton conc. (mmolN/m3)
      real z                   ! IN Zooplankton conc. (mmolN/m3)
      real dn                  ! IN Detritus conc. (mmolN/m3)
      real specific_loss       ! OUT Specific loss (total) (/d)
      logical valid            ! OUT Flag to show calculation completed

! Local variables - model parameters

      real resp_rate           ! Phytoplankton respiration rate (/d)
      real pmort_rate          ! Phytoplankton mortality (/(mmolN/m3)/d) 
      real phyto_min           ! Threshold for phytop. mortality (mmolN/m3)
      real graze_threshold     ! Threshold for grazing func. (mmolN/m3)
      real holling_coef        ! Determines graxing functional form
      real graze_sat           ! Half-sat. constant for grazing (mmolN/m3)
      real graze_max           ! Max specific rate of zoo grazing (/d)
     
! Local variables - working variables

      real work
      real ingestion_rate 
      real specific_graz       
      real specific_mort    

! Get parameter values

      resp_rate=parm(resp_rate_parm)
      pmort_rate=parm(pmort_rate_parm)
      phyto_min=parm(phyto_min_parm)
      graze_threshold=parm(graze_threshold_parm)
      holling_coef=parm(holling_coef_parm)
      graze_sat=parm(graze_sat_parm)
      graze_max=parm(graze_max_parm)

! Calculate food specific grazing rate 

      work=(max(0.0,((n2be_p*p+n2be_d*dn)-graze_threshold))) &
     &     **holling_coef

      ingestion_rate=n2be_z*z*graze_max*work &
     &               /(work+graze_sat**holling_coef)

      work=max(1.0e-10,n2be_p*p+n2be_d*dn)
      specific_graz=ingestion_rate/work

! Calculate P specific mortality rate

      if (p.ge.phyto_min) then
        specific_mort=p*pmort_rate
      else 
        specific_mort=0.0
      endif

! Calculate specific loss for output

      specific_loss=specific_graz+specific_mort+resp_rate

      valid=.true.
      return
      end subroutine p_specific_loss

!!!#endif

!--------------------------------------------------------------------------
END MODULE bioanalysis
